@@ -350,11 +350,11 @@ void UpdateLayout(QTextLayout &layout, const QFont &font,
     layout.setFormats(formats);
 }
 
-void fcitx::FcitxCandidateWindow::updateClientSideUI(
-    const fcitx::FcitxQtFormattedPreeditList &preedit, int cursorpos,
-    const fcitx::FcitxQtFormattedPreeditList &auxUp,
-    const fcitx::FcitxQtFormattedPreeditList &auxDown,
-    const fcitx::FcitxQtStringKeyValueList &candidates, int candidateIndex,
+void FcitxCandidateWindow::updateClientSideUI(
+    const FcitxQtFormattedPreeditList &preedit, int cursorpos,
+    const FcitxQtFormattedPreeditList &auxUp,
+    const FcitxQtFormattedPreeditList &auxDown,
+    const FcitxQtStringKeyValueList &candidates, int candidateIndex,
     int layoutHint, bool hasPrev, bool hasNext) {
     bool preeditVisble = (cursorpos >= 0 || !preedit.isEmpty());
     bool auxUpVisbile = !auxUp.isEmpty();
@@ -408,11 +408,10 @@ void fcitx::FcitxCandidateWindow::updateClientSideUI(
     // intent to use this with wayland. It we have no information above screen
     // edge.
     if (isWayland_) {
-        screenGeometry = QRect(QPoint(0, 0), window->size());
+        screenGeometry = window->frameGeometry();
         cursorRect.translate(window->framePosition());
         auto margins = window->frameMargins();
         cursorRect.translate(margins.left(), margins.top());
-        screenGeometry.translate(window->framePosition());
     } else {
         screenGeometry = window->screen()->geometry();
         auto pos = window->mapToGlobal(cursorRect.topLeft());
@@ -455,42 +454,43 @@ void fcitx::FcitxCandidateWindow::updateClientSideUI(
     show();
 }
 
-} // namespace fcitx
-
-void fcitx::FcitxCandidateWindow::mouseMoveEvent(QMouseEvent *event) {
+void FcitxCandidateWindow::mouseMoveEvent(QMouseEvent *event) {
     bool needRepaint = false;
+
+    bool prevHovered = false;
+    bool nextHovered = false;
     auto oldHighlight = highlight();
     hoverIndex_ = -1;
-    for (int idx = 0, e = candidateRegions_.size(); idx < e; idx++) {
-        if (candidateRegions_[idx].contains(event->pos())) {
-            hoverIndex_ = idx;
-            break;
+
+    prevHovered = prevRegion_.contains(event->pos());
+    if (!prevHovered) {
+        nextHovered = nextRegion_.contains(event->pos());
+        if (!nextHovered) {
+            for (int idx = 0, e = candidateRegions_.size(); idx < e; idx++) {
+                if (candidateRegions_[idx].contains(event->pos())) {
+                    hoverIndex_ = idx;
+                    break;
+                }
+            }
         }
     }
 
-    needRepaint = needRepaint || oldHighlight != highlight();
-
-    auto prevHovered = prevRegion_.contains(event->pos());
-    auto nextHovered = nextRegion_.contains(event->pos());
     needRepaint = needRepaint || prevHovered_ != prevHovered;
-    needRepaint = needRepaint || nextHovered_ != nextHovered;
     prevHovered_ = prevHovered;
+
+    needRepaint = needRepaint || nextHovered_ != nextHovered;
     nextHovered_ = nextHovered;
+
+    needRepaint = needRepaint || oldHighlight != highlight();
     if (needRepaint) {
         renderNow();
     }
 }
 
-void fcitx::FcitxCandidateWindow::mouseReleaseEvent(QMouseEvent *event) {
+void FcitxCandidateWindow::mouseReleaseEvent(QMouseEvent *event) {
     if (event->button() != Qt::LeftButton) {
         return;
     }
-    for (int idx = 0, e = candidateRegions_.size(); idx < e; idx++) {
-        if (candidateRegions_[idx].contains(event->pos())) {
-            Q_EMIT candidateSelected(idx);
-            return;
-        }
-    }
 
     if (prevRegion_.contains(event->pos())) {
         Q_EMIT prevClicked();
@@ -499,10 +499,18 @@ void fcitx::FcitxCandidateWindow::mouseReleaseEvent(QMouseEvent *event) {
 
     if (nextRegion_.contains(event->pos())) {
         Q_EMIT nextClicked();
+        return;
+    }
+
+    for (int idx = 0, e = candidateRegions_.size(); idx < e; idx++) {
+        if (candidateRegions_[idx].contains(event->pos())) {
+            Q_EMIT candidateSelected(idx);
+            return;
+        }
     }
 }
 
-QSize fcitx::FcitxCandidateWindow::sizeHint() {
+QSize FcitxCandidateWindow::sizeHint() {
     auto minH =
         theme_->fontMetrics().ascent() + theme_->fontMetrics().descent();
 
@@ -576,7 +584,7 @@ QSize fcitx::FcitxCandidateWindow::sizeHint() {
     return {static_cast<int>(width), static_cast<int>(height)};
 }
 
-void fcitx::FcitxCandidateWindow::wheelEvent(QWheelEvent *event) {
+void FcitxCandidateWindow::wheelEvent(QWheelEvent *event) {
     if (!theme_ || !theme_->wheelForPaging()) {
         return;
     }
@@ -591,3 +599,5 @@ void fcitx::FcitxCandidateWindow::wheelEvent(QWheelEvent *event) {
         Q_EMIT nextClicked();
     }
 }
+
+} // namespace fcitx
